import os
import shutil
import requests
from PIL import Image
from resizeimage import resizeimage

from .google_api import GoogleCustomSearch


class FetchResizeSave(object):
    """Class with resizing and downloading logic"""

    def __init__(self, developer_key, custom_search_cx):
        self._google_custom_search = GoogleCustomSearch(developer_key,
                                                        custom_search_cx)
        self._search_resut = []

    def search(self, search_params, path_to_dir=False, width=None,
               height=None, cache_discovery=True):
        """Fetched images using Google API and does the download and resize
        if path_to_dir and width and height variables are provided.
        :param search_params: parameters for Google API Search
        :param path_to_dir: path where the images should be downloaded
        :param width: crop width of the images
        :param height: crop height of the images
        :param cache_discovery: whether or not to cache the discovery doc
        :return: None
        """

        for url in self._google_custom_search.search(search_params,
                                                     cache_discovery):
            image = GSImage(self)
            image.url = url

            if path_to_dir:
                image.download(path_to_dir)
                if width and height:
                    image.resize(width, height)

            self._search_resut.append(image)

    def results(self):
        """Returns objects of downloaded images
        :return: list
        """

        return self._search_resut

    def download(self, url, path_to_dir):
        """Downloads image from url to path dir
        Used only by GSImage class
        :param url: image url
        :param path_to_dir: path to directory where image should be saved
        :return: path to image
        """

        if not os.path.exists(path_to_dir):
            os.makedirs(path_to_dir)

        raw_data = self.__class__.get_raw_data(url)

        raw_filename = url.split('/')[-1].split('?')[0]
        basename, ext = os.path.splitext(raw_filename)
        filename = "".join(x for x in basename if x.isalnum()) + ext

        path_to_image = os.path.join(path_to_dir, filename)

        with open(path_to_image, 'wb') as f:
            self.__class__.copy_to(raw_data, f)

        return path_to_image

    @staticmethod
    def get_raw_data(url):
        """Takes data from image url into a variable
        :param url: url to image
        :return: raw image data
        """

        req = requests.get(url, stream=True)
        req.raw.decode_content = True
        return req.raw

    @staticmethod
    def copy_to(raw_data, obj):
        """
        Copy raw image data to another object, preferably BytesIO
        :param raw_data: raw image data
        :param obj: BytesIO object
        :return: None
        """

        shutil.copyfileobj(raw_data, obj)

    @staticmethod
    def resize(path_to_image, width, height):
        """Resize the image and save it again.
        :param path_to_image: os.path
        :param width: int
        :param height: int
        :return: None
        """

        fd_img = open(path_to_image, 'rb')
        img = Image.open(fd_img)
        img = resizeimage.resize_cover(img, [int(width), int(height)])
        img.save(path_to_image, img.format)
        fd_img.close()


class GSImage(object):
    """Class for handling one image"""

    def __init__(self, fetch_resize_save):
        self._fetch_resize_save = fetch_resize_save

        self._url = None
        self._path = None

        self.resized = False

    @property
    def url(self):
        """Returns the image url
        :return: url
        """

        return self._url

    @url.setter
    def url(self, image_url):
        """Sets the image url
        :param image_url: url
        :return: None
        """

        self._url = image_url

    @property
    def path(self):
        """Returns image path
        :return: path
        """

        return self._path

    @path.setter
    def path(self, image_path):
        """Sets image path
        :param image_path: path
        :return: None
        """

        self._path = image_path

    def download(self, path_to_dir):
        """Downloads image from url to path
        :param path_to_dir: path
        :return: None
        """

        self._path = self._fetch_resize_save.download(self._url, path_to_dir)

    def get_raw_data(self):
        """Gets images raw data
        :return: raw data
        """

        return self._fetch_resize_save.__class__.get_raw_data(self._url)

    def copy_to(self, obj, raw_data=None):
        """Copies raw image data to another object, preferably BytesIO
        :param obj: BytesIO
        :param raw_data: raw data
        :return: None
        """

        if not raw_data:
            raw_data = self.get_raw_data()

        self._fetch_resize_save.__class__.copy_to(raw_data, obj)

    def resize(self, width, height):
        """Resize the image
        :param width: int
        :param height: int
        :return: None
        """

        self._fetch_resize_save.__class__.resize(self._path, width, height)
        self.resized = True



import click
from termcolor import cprint
from pyfiglet import figlet_format

from .fetch_resize_save import FetchResizeSave
from .google_api import GoogleBackendException


@click.group()
@click.pass_context
@click.option('-k', '--developer_key', help='Developer API key')
@click.option('-c', '--custom_search_cx', help='Custom Search CX')
def cli(ctx, developer_key, custom_search_cx):
    ctx.obj = {
        'object': FetchResizeSave(developer_key, custom_search_cx)
    }


IMAGE_TYPES = ('clipart', 'face', 'lineart', 'news', 'photo', )
IMAGE_SIZES = ('huge', 'icon', 'large', 'medium', 'small', 'xlarge', 'xxlarge')
FILE_TYPES = ('jpg', 'gif', 'png')
DOMINANT_COLORS = ('black', 'blue', 'brown', 'gray', 'green', 'pink', 'purple',
                   'teal', 'white', 'yellow')
SAFE_SEARCH = ('high', 'medium', 'off', )


@cli.command()
@click.pass_context
@click.option('-q', '--query', help='Search query')
@click.option('-n', '--num', default=5, help='Number of images in response')
@click.option('-s', '--safe', type=click.Choice(SAFE_SEARCH),
              default='off', help='Search safety level')
@click.option('-f', '--filetype', type=click.Choice(FILE_TYPES),
              default='jpg', help='Images file type')
@click.option('-i', '--imagetype', type=click.Choice(IMAGE_TYPES),
              default='photo', help='Image type')
@click.option('-s', '--imagesize', type=click.Choice(IMAGE_SIZES),
              default='large', help='Image size')
@click.option('-c', '--dominantcolor', type=click.Choice(DOMINANT_COLORS),
              default='black', help='Dominant color in images')
@click.option('-d', '--download_path', type=click.Path(dir_okay=True),
              help='Download images')
@click.option('-w', '--width', help='Image crop width')
@click.option('-h', '--height', help='Image crop height')
def search(ctx, query, num, safe, filetype, imagetype,
           imagesize, dominantcolor, download_path, width, height):

    search_params = {
        'q': query,
        'num': num,
        'safe': safe,
        'fileType': filetype,
        'imgType': imagetype,
        'imgSize': imagesize,
        'imgDominantColor': dominantcolor
    }

    click.clear()

    cprint(figlet_format('Google Images Search', width=120), 'red')

    click.echo('-'*120)

    try:
        ctx.obj['object'].search(search_params, download_path, width, height)

        for _, image in enumerate(ctx.obj['object'].results()):
            click.echo(image.url)
            if image.path:
                click.secho(image.path, fg='blue')
                if not image.resized:
                    click.secho('[image is not resized]', fg='red')
            else:
                click.secho('[image is not download]', fg='red')
            click.echo()

    except GoogleBackendException:
        click.secho('Error occurred trying to fetch '
                    'images from Google. Please try again.', fg='red')
        return

    click.echo('-'*120)
    click.echo()



import os
import requests
from apiclient.discovery import build


class GoogleCustomSearch(object):
    """Wrapper class for Google images search api"""

    def __init__(self, developer_key=None,
                 custom_search_cx=None):

        self._developer_key = developer_key or \
                              os.environ.get('GCS_DEVELOPER_KEY')
        self._custom_search_cx = custom_search_cx or \
                                 os.environ.get('GCS_CX')

        self._google_build = None

        self._search_params_keys = {
            'q': None,
            'searchType': 'image',
            'num': 1,
            'imgType': None,
            'imgSize': None,
            'fileType': None,
            'safe': 'off',
            'imgDominantColor': None
        }

    def _query_google_api(self, search_params, cache_discovery=True):
        """Queries Google api
        :param search_params: dict of params
        :param cache_discovery whether or not to cache the discovery doc
        :return: search result object
        """

        if not self._google_build:
            self._google_build = build("customsearch", "v1",
                                       developerKey=self._developer_key,
                                       cache_discovery=cache_discovery)

        return self._google_build.cse().list(
            cx=self._custom_search_cx, **search_params).execute()

    def _search_params(self, params):
        """Received a dict of params and merges
        it with default params dict
        :param params: dict
        :return: dict
        """

        search_params = {}

        for key, value in self._search_params_keys.items():
            params_value = params.get(key)
            if params_value:
                search_params[key] = params_value
            elif value:
                search_params[key] = value

        return search_params

    def search(self, params, cache_discovery=True):
        """Search for images and returns
        them using generator object
        :param params: search params
        :param cache_discovery whether or not to cache the discovery doc
        :return: yields url to searched image
        """

        search_params = self._search_params(params)

        try:
            res = self._query_google_api(search_params, cache_discovery)
        except:
            raise GoogleBackendException()

        for image in res.get('items'):
            try:
                check = requests.get(image['link'], timeout=5)
                if check.status_code == 200:
                    yield image['link']
            except requests.exceptions.ConnectTimeout:
                pass


class GoogleBackendException(Exception):
    """Exception handler for search api"""



from setuptools import setup


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='Google Images Search',
    version="0.3.8",

    description='Search for image using Google Custom Search API and resize & crop the image afterwords',
    long_description=readme(),
    long_description_content_type='text/markdown',

    url='https://github.com/arrrlo/Google-Images-Search',
    licence='MIT',

    author='Ivan Arar',
    author_email='ivan.arar@gmail.com',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='google images, resize, crop',

    packages=['google_images_search'],
    install_requires=[
        'colorama~=0.4',
        'pyfiglet~=0.8',
        'termcolor~=1.1',
        'click~=7.0',
        'six~=1.12',
        'requests~=2.21',
        'Pillow~=6.0',
        'python-resize-image~=1.1',
        'google-api-python-client~=1.7',
    ],

    entry_points={
        'console_scripts': [
            'gimages=google_images_search.cli:cli'
        ],
    },

    project_urls={
        'Source': 'https://github.com/arrrlo/Google-Images-Search',
    },
)
